<?php
/**
 * @name      OpenImporter
 * @copyright OpenImporter contributors
 * @license   BSD http://opensource.org/licenses/BSD-3-Clause
 *
 * @version 1.0 Alpha
 *
 * This file contains code based on:
 *
 * Simple Machines Forum (SMF)
 * copyright:	2011 Simple Machines (http://www.simplemachines.org)
 * license:	BSD, See included LICENSE.TXT for terms and conditions.
 *
 *	Updated by ian for http://dangerousprototypes.com
 *	22 August 2019
 */


if (!defined('DS'))
	define('DS', DIRECTORY_SEPARATOR);

/**
 * Object Importer (used to) creates the main XML object.
 * It detects and initializes the script to run.
 *	...Now it's been decimated and feasted on by wild dogs so we can convert the forum at dangerousprototypes.com
 * @package OpenImporter
 */
class Importer
{
	/**
	 * This is our main database object.
	 * @var Database
	 */
	protected $db;

	public function process(){
		$db_server='localhost';
		$db_user='dpforum2';
		$db_passwd='12345';
		$db_persist=true;
		
		$to_prefix ='`dpforum2`.elkarte_';
		$from_prefix='`dpforum2`.phpbb_';
		$from_path='C:\laragon\www\forum';
		$to_path='C:\laragon\www\jsplumb';
		$my_member_name='ian'; //to reset your password to 12345 for easy testing
		
		$this->init_db($db_server, $db_user, $db_passwd, $db_persist);
		
		//steps...
		$this->import_ranks($from_prefix, $to_prefix);
		$this->import_groups($from_prefix, $to_prefix);
		$this->import_members($from_prefix, $to_prefix, $from_path, $to_path);
		
		//this just sets your membername password to 12345 for easy debugging (dont have to reset your password manually each time)
		$this->set_my_password_to_12345($my_member_name, $to_prefix);

		$this->import_membergroups($from_prefix, $to_prefix);
		$this->import_banned($from_prefix, $to_prefix);
		$this->import_categories($from_prefix, $to_prefix);
		
		//boards will handle main boards and first layer of subboards only
		//boards 2 levels deep and lower will go into a new "MANUALLY SORT ME" category
		$this->import_boards($from_prefix, $to_prefix);
		
		$this->import_topics($from_prefix, $to_prefix);
		$this->import_posts($from_prefix, $to_prefix);
		$this->import_polls($from_prefix, $to_prefix);
		$this->import_personalmessages($from_prefix, $to_prefix); 
		$this->import_subscriptions($from_prefix, $to_prefix);
		
		// warning about attachments: we do not attempt to clean up the folder, 
		// so if you run multiple times be sure to clean out the attachments folder before each run
		//post attachments
		$this->import_attachments($from_prefix, $to_prefix, $from_path, $to_path, $pm=false);
		//PM attachments
		$this->import_attachments($from_prefix, $to_prefix, $from_path, $to_path, $pm=true);
		
		$this->import_cleanup($from_prefix, $to_prefix);
		
		die();
	}

	protected function init_db($db_server, $db_user, $db_passwd, $db_persist)
	{
		$this->db = new Database($db_server, $db_user, $db_passwd, $db_persist);

		// We want UTF8 only, let's set our mysql connection to utf8
		$this->db->query('SET NAMES \'utf8\'');

		// SQL_BIG_SELECTS: If set to 0, MySQL aborts SELECT statements that are
		// likely to take a very long time to execute (that is, statements for
		// which the optimizer estimates that the number of examined rows exceeds
		// the value of max_join_size)
		// Source:
		// https://dev.mysql.com/doc/refman/5.5/en/server-system-variables.html#sysvar_sql_big_selects
		$this->db->query("SET @@SQL_BIG_SELECTS = 1");
		$this->db->query("SET @@MAX_JOIN_SIZE = 18446744073709551615");
	}	
	
	function set_my_password_to_12345($member_name, $to_prefix){
		$this->db->query('UPDATE '.$to_prefix.'members SET passwd="$2a$10$Rmlhg4T3xj7mXW1tyXbCheNn.o90Ntl0nYuhRRsoft3hoZVAF7ddO" WHERE member_name="'.$member_name.'" LIMIT 1');
	}
		
	function import_ranks($from_prefix, $to_prefix){
		//Converting ranks...
		$this->db->query("DELETE FROM {$to_prefix}membergroups
		WHERE group_name LIKE 'phpBB %';");
		// Got some ranks to move, so go do it... first remove the post based membergroups from the table.
		// !!! We must keep group id 4 as a post group.  MUST!!
		$this->db->query("DELETE FROM {$to_prefix}membergroups
			WHERE min_posts != -1
				AND id_group > 4");
		$query="SELECT
				SUBSTRING(CONCAT('phpBB ', rank_title), 1, 255) AS group_name,
				rank_image AS icons, '' AS description, IF(rank_special = 0, rank_min, -1) AS min_posts,
				'' AS online_color
			FROM {$from_prefix}ranks
			ORDER BY rank_min;"; //NOTE: rank_image AS stars for SMF, icons for elkarte
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		$convert_num_stars = 1;
		foreach($rows as $row){
			
			if ($row['min_posts'] > -1) 
			{
				$row['icons'] = sprintf("%d#icon.gif", $convert_num_stars);//NOTE: rank_image AS stars for SMF, icons for elkarte
				if ($convert_num_stars < 5)
					$convert_num_stars++;
			}
			//print_r($row);
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		//print_r($insert_rows);
		$query="INSERT IGNORE
				INTO   {$to_prefix}membergroups
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);	
	}		

	function import_groups($from_prefix, $to_prefix){
		//Converting groups...
		$this->db->query("TRUNCATE {$to_prefix}membergroups;");	
		$query="SELECT
			SUBSTRING(CONCAT('phpBB ', group_name), 1, 255) AS group_name,
			-1 AS min_posts, '' AS icons, '' AS description, group_colour AS online_color
		FROM {$from_prefix}groups
		WHERE group_id NOT IN (1, 6);"; //EXPERIMENTAL: WHERE group_id NOT IN (1, 6);";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}membergroups
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),(', $insert_rows) . ")";		
		$this->db->query($query);	
	}

	function import_members($from_prefix, $to_prefix, $from_path, $to_path){
		// Got the board timezone?
		$board_timezone=$this->get_phpbb_config($from_prefix,'board_timezone');
		// Find out where uploaded avatars go
		$custom_avatar_enabled=$this->get_elkarte_config($to_prefix,'custom_avatar_enabled');
		if ($custom_avatar_enabled)
		{
			// Custom avatar dir.
			$avatar_dir = $this->get_elkarte_config($to_prefix,'custom_avatar_dir');
			$attachment_type = '1';
						//this setting variable looks like a:1:{i:1;s:34:"C:\laragon\www\jsplumb/attachments";}, clean it up!
			$clean=explode('"',$avatar_dir);
			$avatar_dir=$clean[1]; //this will error big if your settings dont look like this	
		}
		else
		{
			// Attachments dir.
			$avatar_dir = $this->get_elkarte_config($to_prefix,'attachmentUploadDir');
			$attachment_type = '0';
									//this setting variable looks like a:1:{i:1;s:34:"C:\laragon\www\jsplumb/attachments";}, clean it up!
			$clean=explode('"',$avatar_dir);
			$avatar_dir=$clean[1]; //this will error big if your settings dont look like this	
		}
		$phpbb_avatar_upload_path = $from_path . '/' . $this->get_phpbb_config($from_prefix,'avatar_path');
		//die($phpbb_avatar_upload_path);
		//echo $phpbb_avatar_upload_path;
		$phpbb_avatar_salt = $this->get_phpbb_config($from_prefix,'avatar_salt');
		//die($phpbb_avatar_salt);
		
		//import members
		//applied banned member fix
		// added u.user_sig_bbcode_uid AS signature_uid
		$this->db->query("TRUNCATE {$to_prefix}members");
		$this->db->query("TRUNCATE {$to_prefix}attachments");
		$query="SELECT
			u.user_id AS id_member, 
			SUBSTRING(u.username, 1, 80) AS member_name,
			SUBSTRING(u.username, 1, 255) AS real_name,
			SUBSTRING(u.user_password, 1, 64) AS passwd, u.user_lastvisit AS last_login,
			u.user_regdate AS date_registered, '' AS buddy_list,
			SUBSTRING(u.user_from, 1, 255) AS location,
			u.user_posts AS posts, IF(u.user_rank = 1, 1, IFNULL(mg.id_group, 0)) AS id_group,
			u.user_new_privmsg AS unread_messages, '' AS message_labels,
			SUBSTRING(u.user_email, 1, 255) AS email_address,
			u.user_unread_privmsg AS unread_messages, '' AS ignore_boards,
			SUBSTRING(u.user_website, 1, 255) AS website_title,
			SUBSTRING(u.user_website, 1, 255) AS website_url,
			u.user_allow_viewonline AS show_online, u.user_timezone AS time_offset,
			IF(u.user_allow_viewemail = 1, 0, 1) AS hide_email, u.user_avatar AS avatar,
			REPLACE(u.user_sig, '\n', '<br />') AS signature, u.user_sig_bbcode_uid AS tmp_bbc_replace,u.user_sig_bbcode_uid AS signature_uid, u.user_avatar_type,
			u.user_notify_pm AS pm_email_notify,
			CASE u.user_inactive_reason WHEN 0 THEN 1 ELSE 0 END AS is_activated,
			u.user_ip AS member_ip
		FROM {$from_prefix}users AS u
			LEFT JOIN {$from_prefix}ranks AS r ON (r.rank_id = u.user_rank AND r.rank_special = 1)
			LEFT JOIN {$to_prefix}membergroups AS mg ON (mg.group_name = CONCAT('phpBB ', r.rank_title))
		WHERE u.group_id <> 6 AND user_id > 1
		GROUP BY u.user_id";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			//processing loop!
			// time_offset = phpBB user TZ - phpBB board TZ.
			$row['time_offset'] = $row['time_offset'] - $board_timezone;

			//no avatar
			if ($row['user_avatar_type'] == 0){
				$row['avatar'] = '';
				//echo "no avitar";
			// If the avatar type is uploaded (type = 1) copy avatar with the correct name.
			//missing avatar update
			}elseif ($row['user_avatar_type'] == 1 && strlen($row['avatar']) > 0)
			{
				$phpbb_avatar_ext = substr(strchr($row['avatar'], '.'), 1);
				$smf_avatar_filename = 'avatar_' . $row['id_member'] . strrchr($row['avatar'], '.');
				//echo $phpbb_avatar_upload_path . '/' . $phpbb_avatar_salt . '_' . $row['id_member'] . '.' . $phpbb_avatar_ext;
				if (file_exists($phpbb_avatar_upload_path . '/' . $phpbb_avatar_salt . '_' . $row['id_member'] . '.' . $phpbb_avatar_ext))
				{
					copy($phpbb_avatar_upload_path . '/' . $phpbb_avatar_salt . '_' . $row['id_member'] . '.' . $phpbb_avatar_ext, $avatar_dir . '/' . $smf_avatar_filename);
					$this->db->query("INSERT IGNORE INTO {$to_prefix}attachments (id_msg, id_member, filename, attachment_type) 
					VALUES(0, '".$row['id_member']."','".substr(addslashes($smf_avatar_filename), 0, 255)."','{$attachment_type}');");
					//echo "added file<br>";
					
				}
				elseif (file_exists($phpbb_avatar_upload_path . '/' . $row['avatar']))//I HAVE NOT TESTED THIS METHOD....
				{
					copy($phpbb_avatar_upload_path . '/' . $row['avatar'], $avatar_dir . '/' . $smf_avatar_filename);
					$this->db->query("INSERT IGNORE INTO {$to_prefix}attachments (id_msg, id_member, filename, attachment_type) 
					VALUES (0, '".$row['id_member']."','".substr(addslashes($smf_avatar_filename), 0, 255)."','{$attachment_type}');");
					//echo "added file<br>";
				}else{
					//echo "didn't find avatar file<br>";
				}
				$row['avatar'] = '';
			}
			elseif ($row['user_avatar_type'] == 3){//gallery avatar?
				$row['avatar'] = substr('gallery/' . $row['avatar'], 0, 255);
				//echo "avatar type 3"; //I didnt have any so I dont know if this works or not
			}
			
			unset($row['user_avatar_type']);

			if ($row['signature_uid'] != '')
				$row['signature'] = preg_replace('~(:u:|:1:|:)' . preg_quote($row['signature_uid'], '~') . '~i', '', $row['signature']);

			$row['signature'] = $this->phpbb_replace_bbc($row['signature']);
			$row['signature'] = str_replace($row['tmp_bbc_replace'], '', $row['signature']);
			unset($row['tmp_bbc_replace']);	
			$row['signature'] = substr($row['signature'], 0, 65534);
			unset($row['signature_uid']);
			if (!is_numeric($row['id_group']))
				$row['id_group'] = 0;
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";

		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}members
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);
	}
	

	// Converting additional member groups...
	function import_membergroups($from_prefix, $to_prefix){	

		$result = $this->db->query("
			SELECT mg.id_group, mem.id_member
			FROM {$from_prefix}groups AS g
				INNER JOIN {$from_prefix}user_group AS ug ON (ug.group_id = g.group_id)
				INNER JOIN {$to_prefix}members AS mem ON (mem.id_member = ug.user_id)
				INNER JOIN {$to_prefix}membergroups AS mg ON (mg.group_name = CONCAT('phpBB ', g.group_name))
			WHERE g.group_name NOT IN ('GUESTS', 'REGISTERED_COPPA', 'BOTS')
			ORDER BY id_member");
		$additional_groups = '';
		$last_member = 0;
		while ($row = $result->fetch_assoc())
		{
			if (empty($last_member))
				$last_member = $row['id_member'];

			if ($last_member != $row['id_member'])
			{
				$additional_groups = addslashes($additional_groups);

				$this->db->query("
					UPDATE {$to_prefix}members
					SET additional_groups = '$additional_groups'
					WHERE id_member = $last_member
					LIMIT 1");
				$last_member = $row['id_member'];
				$additional_groups = $row['id_group'];
			}
			else
			{
				if ($additional_groups == '')
					$additional_groups = $row['id_group'];
				else
					$additional_groups = $additional_groups . ',' . $row['id_group'];
			}
		}


		if ($last_member != 0)
		{
			$additional_groups = addslashes($additional_groups);

			$this->db->query("
				UPDATE {$to_prefix}members
				SET additional_groups = '$additional_groups'
				WHERE id_member = $last_member
				LIMIT 1");
		}
	}


	function import_banned($from_prefix, $to_prefix){		
		
		//Import banned members
		$this->db->query("TRUNCATE {$to_prefix}ban_groups");
		$this->db->query("TRUNCATE {$to_prefix}ban_items;");
		$query="SELECT
			  b.ban_id AS id_ban_group, m.username AS name,
			  b.ban_start AS ban_time, b.ban_end as expire_time,
			  b.ban_give_reason AS reason, b.ban_reason AS notes,
			  SUBSTRING(m.user_email, 1, 255) as email_address, b.ban_userid AS id_member
			FROM {$from_prefix}banlist AS b
				LEFT JOIN {$from_prefix}users AS m ON (m.user_id = b.ban_userid)
			ORDER BY b.ban_id ASC;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$row['cannot_access'] = 1;
			$row['cannot_login'] = 1;
			$this->db->query("INSERT IGNORE INTO {$to_prefix}ban_items (id_ban_group, id_member) VALUES
				('{$row['id_ban_group']}','{$row['id_member']}')");
			unset($row['id_member']);
			$this->db->query("INSERT IGNORE INTO {$to_prefix}ban_items (id_ban_group, email_address) VALUES
				('{$row['id_ban_group']}','".addslashes($row['email_address'])."')");
			unset($row['email_address']);
			if ($row['expire_time'] == 0)
			  unset($row['expire_time']);
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}ban_groups
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);		

		$this->db->query("ALTER TABLE {$to_prefix}ban_groups ORDER BY id_ban_group");
		$this->db->query("ALTER TABLE {$to_prefix}ban_items ORDER BY id_ban;");
	}	
		
	function import_categories($from_prefix, $to_prefix){
		//Importing Categories
		$this->db->query("TRUNCATE {$to_prefix}categories;");	
		$query="SELECT
				forum_id AS id_cat, SUBSTRING(forum_name, 1, 255) AS name, left_id AS cat_order
			FROM {$from_prefix}forums
			WHERE forum_type = 0;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}categories
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);
	}

	function import_boards($from_prefix, $to_prefix){
		//I'm not going to do any handling of sub forms, it does not seem to work in any version of any script I've seen....
		//so we make a category to organize manually after the conversion....
		$this->db->query("INSERT INTO {$to_prefix}categories (name, cat_order) VALUES ('SUBFORUMS MANUALLY SORT, SORRY!', 1)");
		$subform_category=$this->db->insert_id();
		
		//Importing Boards
		$this->db->query("TRUNCATE {$to_prefix}boards;");
		$this->db->query("DELETE FROM {$to_prefix}board_permissions WHERE id_profile > 4;");
		$query="SELECT
				f.forum_id AS id_board, CASE WHEN f.parent_id = c.id_cat THEN 0 ELSE f.parent_id END AS id_parent, f.left_id AS board_order,
				f.forum_posts AS num_posts, f.forum_last_post_id AS id_last_msg,
				SUBSTRING(f.forum_name, 1, 255) AS name, 
				IFNULL(c.id_cat,{$subform_category}) AS id_cat, 
				'-1,0' AS member_groups,
				SUBSTRING(f.forum_desc, 1, 65534) AS description, f.forum_topics AS num_topics,
				f.forum_last_post_id AS id_last_msg
			FROM {$from_prefix}forums AS f
				LEFT JOIN {$to_prefix}categories AS c ON (c.id_cat = f.parent_id)
			WHERE forum_type = 1
			GROUP BY id_board;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			//if (empty($row['id_cat']))
				//$row['id_cat'] = 1;			
			$row['name'] = str_replace('\n', '<br />', $row['name']);
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}boards
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);	

		//fix the first layer of subboards
		//else assign to our new "FIX ME MANUALLY" category
		foreach($rows as $row){
			if($row['id_parent']!=0){
				$result=$this->db->query("SELECT id_cat FROM {$to_prefix}boards WHERE id_board={$row['id_parent']}");
				$rows=$result->fetch_all(MYSQLI_ASSOC);
				if(count($rows)>0){ //if we find the parent we can change to it's category ID and it will be visible
					$id_cat=$rows[0]['id_cat'];
				}else{//if not we put it in the new category so it is also visible and we can fix it later
					$id_cat=$subform_category;
				}
				$this->db->query("UPDATE {$to_prefix}boards SET id_cat='{$id_cat}', child_level=1 WHERE id_board='{$row['id_board']}'");
			}
		}
		
	}
	
	function import_topics($from_prefix, $to_prefix){
		//Importing Topics
		$this->db->query("TRUNCATE {$to_prefix}topics;");
		$this->db->query("TRUNCATE {$to_prefix}log_topics;");
		$this->db->query("TRUNCATE {$to_prefix}log_boards;");
		$this->db->query("TRUNCATE {$to_prefix}log_mark_read;");
		
		$query="SELECT
				t.topic_id AS id_topic, t.forum_id AS id_board, t.topic_first_post_id AS id_first_msg,
				CASE t.topic_type
					WHEN 1 THEN 1
					WHEN 2 THEN 1
					ELSE 0
				END AS is_sticky,
				t.topic_last_post_id AS id_last_msg, t.topic_poster AS id_member_started,
				t.topic_last_poster_id AS id_member_updated, IF(po.topic_id > 0, po.topic_id, 0) AS id_poll,
				t.topic_replies AS num_replies, t.topic_views AS num_views,
				CASE t.topic_status
					WHEN 1 THEN 1
					ELSE 0
				END AS locked
			FROM {$from_prefix}topics AS t
				LEFT JOIN {$from_prefix}poll_options AS po ON (po.topic_id = t.topic_id);";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}topics
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);					
	}
	
	function import_posts($from_prefix, $to_prefix){	
	//Importing posts
		$this->db->query("TRUNCATE {$to_prefix}messages;");
		$query="SELECT
				p.post_id AS id_msg, p.topic_id AS id_topic, p.forum_id AS id_board,
				p.post_time AS poster_time, p.poster_id AS id_member, p.post_subject AS subject,
				IFNULL(m.username, 'Guest') AS poster_name,	IFNULL(m.user_email, 'Unknown') AS poster_email,
				IFNULL(p.poster_ip, '0.0.0.0') AS poster_ip, p.enable_smilies AS smileys_enabled,
				IFNULL(p.post_edit_time,0) AS modified_time, p.bbcode_uid AS tmp_bbc_replace,
				IFNULL(CASE p.post_edit_user WHEN 0 THEN 'Guest' ELSE m2.username END, 0) AS modified_name,
				p.post_text AS body
			FROM {$from_prefix}posts AS p
				LEFT JOIN {$from_prefix}users AS m ON (m.user_id = p.poster_id)
				LEFT JOIN {$from_prefix}users AS m2 ON (m2.user_id = p.post_edit_user);";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$row['body'] = $this->phpbb_replace_bbc($row['body']);
			$row['body'] = str_replace($row['tmp_bbc_replace'], '', $row['body']);
			unset($row['tmp_bbc_replace']);			
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}messages
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);			
	}	
		
	function import_polls($from_prefix, $to_prefix){	
		//Importing polls
		$this->db->query("TRUNCATE {$to_prefix}polls;");
		$this->db->query("TRUNCATE {$to_prefix}poll_choices;");
		$this->db->query("TRUNCATE {$to_prefix}log_polls;");
		
		$query="SELECT
				t.topic_id AS id_poll, t.poll_title AS question, t.poll_max_options AS max_votes,
				IFNULL((t.poll_start + t.poll_length), 0) AS expire_time, t.poll_vote_change AS change_vote,
				t.topic_poster AS id_member, IFNULL(m.username, 0) AS poster_name
			FROM {$from_prefix}topics AS t
				LEFT JOIN {$from_prefix}users AS m ON (m.user_id = t.topic_poster)
			WHERE t.poll_title != '';";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}polls
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);				

		//Importing poll choices
		$query="SELECT
				topic_id AS id_poll, poll_option_id AS id_choice,
				SUBSTRING(poll_option_text, 1, 255) AS label, poll_option_total AS votes
			FROM {$from_prefix}poll_options;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}poll_choices
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);				
			
		//Importing poll votes
		$query="SELECT
				topic_id AS id_poll, vote_user_id AS id_member, poll_option_id AS id_choice
			FROM {$from_prefix}poll_votes
			WHERE vote_user_id > 0;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}log_polls
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);	
	}

	function import_personalmessages($from_prefix, $to_prefix){	
		// Preparing for PM conversion...
		$this->db->query("DROP TABLE IF EXISTS {$to_prefix}pm_attachments;");
		$this->db->query("CREATE TABLE IF NOT EXISTS {$to_prefix}pm_attachments (
		  `id_attach` int(10) unsigned NOT NULL AUTO_INCREMENT,
		  `id_thumb` int(10) unsigned NOT NULL DEFAULT '0',
		  `id_pm` int(10) unsigned NOT NULL DEFAULT '0',
		  `pm_report` int(10) unsigned NOT NULL DEFAULT '0',
		  `id_folder` tinyint(3) NOT NULL DEFAULT '1',
		  `attachment_type` tinyint(3) unsigned NOT NULL DEFAULT '0',
		  `filename` tinytext NOT NULL,
		  `file_hash` varchar(40) NOT NULL DEFAULT '',
		  `fileext` varchar(8) NOT NULL DEFAULT '',
		  `size` int(10) unsigned NOT NULL DEFAULT '0',
		  `downloads` mediumint(8) unsigned NOT NULL DEFAULT '0',
		  `width` mediumint(8) unsigned NOT NULL DEFAULT '0',
		  `height` mediumint(8) unsigned NOT NULL DEFAULT '0',
		  `mime_type` varchar(20) NOT NULL DEFAULT '',
		  PRIMARY KEY (`id_attach`),
		  KEY `id_pm` (`id_pm`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;");
	
		//Importing personal messages
		//grouped messages fix
		$this->db->query("TRUNCATE {$to_prefix}personal_messages;");
		$this->db->query("TRUNCATE {$to_prefix}pm_recipients;");
		$query="SELECT
				pm.msg_id AS id_pm, pm.author_id AS id_member_from, pm.message_time AS msgtime,
				SUBSTRING(uf.username, 1, 255) AS from_name, SUBSTRING(pm.message_subject, 1, 255) AS subject,
				pm.bbcode_uid AS tmp_bbc_replace,
				SUBSTRING(REPLACE(IF(pm.bbcode_uid = '', pm.message_text,
				REPLACE(REPLACE(pm.message_text, CONCAT(':1:', pm.bbcode_uid), ''), CONCAT(':', pm.bbcode_uid), '')), '\n', '<br>'), 1, 65534) AS body,
				CASE pm.root_level WHEN 0 THEN pm.msg_id ELSE pm.root_level END AS id_pm_head
			FROM {$from_prefix}privmsgs AS pm
				LEFT JOIN {$from_prefix}users AS uf ON (uf.user_id = pm.author_id);";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$row['body'] = $this->phpbb_replace_bbc($row['body']);
			$row['body'] = str_replace($row['tmp_bbc_replace'], '', $row['body']);
			unset($row['tmp_bbc_replace']);			
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}personal_messages
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);	
			
			
		//Importing pm recipients
		$query="SELECT
			pm.msg_id AS id_pm, pm.user_id AS id_member, '-1' AS labels,
			CASE pm.pm_unread WHEN 1 THEN 0 ELSE 1 END AS is_read, pm.pm_deleted AS deleted
		FROM {$from_prefix}privmsgs_to AS pm
		GROUP BY pm.msg_id;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){	
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}pm_recipients
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);			
	}

	function import_subscriptions($from_prefix, $to_prefix){	
			
		//Converting topic subscriptions....
		$this->db->query("TRUNCATE {$to_prefix}log_topics;");
		$query="SELECT
			  w.topic_id AS id_topic, w.user_id AS id_member,
			  IFNULL(t.topic_last_post_id, 0) AS id_msg
			FROM {$from_prefix}topics_watch AS w
			  LEFT JOIN {$from_prefix}topics AS t ON (t.topic_id = w.topic_id)
			ORDER BY w.topic_id ASC;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}log_topics
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);			

		//Converting board subscriptions....
		$this->db->query("TRUNCATE {$to_prefix}log_boards;");
		$query="SELECT
		  w.forum_id AS id_board, w.user_id AS id_member,
		  IFNULL(f.forum_last_post_id, 0) AS id_msg
		FROM {$from_prefix}forums_watch AS w
		  LEFT JOIN {$from_prefix}forums AS f ON (f.forum_id = w.forum_id)
		ORDER BY w.forum_id ASC;";
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);
		$insert_rows=[];
		foreach($rows as &$row){
			$insert_rows[] = "'" . implode("', '", addslashes_recursive($row)) . "'";
		}
		$query="INSERT IGNORE
				INTO   {$to_prefix}log_boards
				(" . implode(', ', array_keys($rows[0])) . ")
			VALUES (" . implode('),
				(', $insert_rows) . ")";		
		$this->db->query($query);	
	}

	// Converting attachments ($pm=true for private message attachments)...
	function import_attachments($from_prefix, $to_prefix, $from_path, $to_path,$pm=false){
		//Converting attachments...
		//get paths
		if(!$pm){
			//$this->db->query("TRUNCATE {$to_prefix}attachments;"); //don't do this here, because we've already added stuff for avitars... 
			$attachmentUploadDir=$this->get_elkarte_config($to_prefix,'attachmentUploadDir');
			//this setting variable looks like a:1:{i:1;s:34:"C:\laragon\www\jsplumb/attachments";}, clean it up!
			$clean=explode('"',$attachmentUploadDir);
			$attachmentUploadDir=$clean[1]; //this will error big if your settings dont look like this		
			$rows=$this->db->query("
				SELECT MAX(id_attach) + 1 as id_attach
				FROM {$to_prefix}attachments");			
		}else{
			
			$attachmentUploadDir = $to_path . '/pm_attachments';
			if(!file_exists($attachmentUploadDir)) //just make it!
				mkdir($attachmentUploadDir, 0777, true);
			$this->db->query("TRUNCATE {$to_prefix}pm_attachments");
			$rows=$this->db->query("
				SELECT MAX(id_attach) + 1 as id_attach
				FROM {$to_prefix}pm_attachments");	
		}		
		$result=$rows->fetch_all(MYSQLI_ASSOC);
		$id_attach=$result[0]['id_attach'];	
		$id_attach = empty($id_attach) ? 1 : $id_attach;	
			
		$oldAttachmentDir=$this->get_phpbb_config($from_prefix,'upload_path');	
		$oldAttachmentDir = $from_path . '/' . $oldAttachmentDir;	

		//add files
		if(!$pm){//post attachments
			// Get $id_attach.
			$rows=$this->db->query("
				SELECT MAX(id_attach) + 1 as id_attach
				FROM {$to_prefix}attachments");

			$query="SELECT
						post_msg_id AS id_msg, download_count AS downloads,
						real_filename AS filename, physical_filename, filesize AS size
					FROM {$from_prefix}attachments
					WHERE topic_id <> 0
					ORDER BY post_msg_id ASC"; //updated to sort PM and post attachments
		}else{	//attachments to private messages
			$query="SELECT
						at.post_msg_id AS id_pm, at.download_count AS downloads,
						at.real_filename AS filename, at.physical_filename, at.filesize AS size
					FROM {$from_prefix}attachments AS at
						LEFT JOIN {$from_prefix}privmsgs AS pm ON ( at.post_msg_id = pm.msg_id )
					WHERE at.topic_id = 0
					ORDER BY at.post_msg_id ASC";
		}
			
		$result=$this->db->query($query);
		$rows=$result->fetch_all(MYSQLI_ASSOC);

		$insert_rows=[];
		foreach($rows as &$row){

			// Set the default empty values.
			$width = 0;
			$height = 0;

			// Is an an image?
			$attachmentExtension = strtolower(substr(strrchr($row['filename'], '.'), 1));
			if (!in_array($attachmentExtension, array('jpg', 'jpeg', 'gif', 'png', 'bmp')))
				$attachmentExtension = '';
			
			$file_hash=sha1(md5($row['filename'] . time()) . mt_rand());			
			//$file_hash = getAttachmentFilename($row['filename'], $id_attach, null, true);
			$physical_filename = $id_attach . '_' . $file_hash.'.elk';

			if (strlen($physical_filename) > 255)
				return;
			if(!file_exists($oldAttachmentDir . '/' . $row['physical_filename'])){
				echo "Missing file: {$row['physical_filename']}<br>\n";
			}
			elseif (copy($oldAttachmentDir . '/' . $row['physical_filename'], $attachmentUploadDir . '/' . $physical_filename))
			{
				// Is an an image?
				if (!empty($attachmentExtension))
				{
					list ($width, $height) = getimagesize($attachmentUploadDir . '/' . $physical_filename);
					// This shouldn't happen but apparently it might
					// IAN: best comment in this code...
					if(empty($width))
						$width = 0;
					if(empty($height))
						$height = 0;
				}
				$insert = array(
					'id_attach' => $id_attach,
					'size' => filesize($attachmentUploadDir . '/' . $physical_filename),
					'filename' => $row['filename'],
					'file_hash' => $file_hash,
					'downloads' => $row['downloads'],
					'width' => $width,
					'height' => $height,
					//'id_pm'=> $row['id_pm']
				);
				if(!$pm){ //keep in mind we cant mix and match post and pm attachments or the insert will be all crazy messed up....
					$insert['id_msg'] = $row['id_msg'];
				}else{
					$insert['id_pm'] = $row['id_pm'];
				}
	
				$id_attach++;
				$insert_rows[] = "'" . implode("', '", addslashes_recursive($insert)) . "'";
			}else{
				echo "Error on file: {$id_attach}<br>\n";
			}
			
			
			//die($physical_filename);
		}
		
		if(isset($insert)){//a scant of hacky protection if there are no files of this type...
			
			if(!$pm){
				$query="INSERT IGNORE
						INTO   {$to_prefix}attachments
						(" . implode(', ', array_keys($insert)) . ")
					VALUES (" . implode('),
						(', $insert_rows) . ")";		
			}else{
				$query="INSERT IGNORE
					INTO   {$to_prefix}pm_attachments
					(" . implode(', ', array_keys($insert)) . ")
				VALUES (" . implode('),
					(', $insert_rows) . ")";	
			}
			$this->db->query($query);	
		}
	}

	function import_cleanup($from_prefix, $to_prefix){	
		//remove double slashes....
		$this->db->query('UPDATE '.$to_prefix.'boards SET description = REPLACE(description, "\\\'", "\'");');
		$this->db->query('UPDATE '.$to_prefix.'messages SET body = REPLACE(body, "\\\'", "\'");');
		$this->db->query('UPDATE '.$to_prefix.'personal_messages SET body = REPLACE(body, "\\\'", "\'");');
		
		$this->db->query("INSERT INTO {$to_prefix}smileys (`code`, `filename`, `description`, `smiley_row`, `smiley_order`, `hidden`) VALUES
		(':-D', 'cheesy.gif', 'Cheesy', 0, 0, 1),
		(':grin:', 'cheesy.gif', 'Cheesy', 0, 0, 1),
		(':-)', 'smiley.gif', 'Smiley', 0, 0, 1),
		(':smile:', 'smiley.gif', 'Smiley', 0, 0, 1),
		(';-)', 'wink.gif', 'Wink', 0, 0, 1),
		(':wink:', 'wink.gif', 'Wink', 0, 0, 1),
		(':-(', 'sad.gif', 'Sad', 0, 0, 1),
		(':sad:', 'sad.gif', 'Sad', 0, 0, 1),
		(':-o', 'shocked.gif', 'Shocked', 0, 0, 1),
		(':eek:', 'shocked.gif', 'Shocked', 0, 0, 1),
		(':shock:', 'shocked.gif', 'Shocked', 0, 0, 1),
		(':???:', 'huh.gif', 'Huh?', 0, 0, 1),
		(':?', 'huh.gif', 'Huh?', 0, 0, 1),
		(':-?', 'huh.gif', 'Huh?', 0, 0, 1),
		('8-)', 'cool.gif', 'Cool', 0, 0, 1),
		(':cool:', 'cool.gif', 'Cool', 0, 0, 1),
		(':lol:', 'laugh.gif', 'Laugh', 0, 0, 1),
		(':x', 'grin.gif', 'Grin', 0, 0, 1),
		(':-x', 'grin.gif', 'Grin', 0, 0, 1),
		(':mad:', 'grin.gif', 'Grin', 0, 0, 1),
		(':-P', 'tongue.gif', 'Tongue', 0, 0, 1),
		(':razz:', 'tongue.gif', 'Tongue', 0, 0, 1),
		(':oops:', 'embarrassed.gif', 'Embarrassed', 0, 0, 1),
		(':cry:', 'cry.gif', 'Cry', 0, 0, 1),
		(':evil:', 'evil.gif', 'Evil', 0, 0, 1),
		(':roll:', 'rolleyes.gif', 'Roll Eyes', 0, 0, 1);");			
	}	
	
	function phpbb_replace_bbc($message)
	{
		$message = preg_replace(
			array(
				'~\[quote=&quot;(.+?)&quot;\:(.+?)\]~is',
				'~\[quote\:(.+?)\]~is',
				'~\[/quote\:(.+?)\]~is',
				'~\[b\:(.+?)\]~is',
				'~\[/b\:(.+?)\]~is',
				'~\[i\:(.+?)\]~is',
				'~\[/i\:(.+?)\]~is',
				'~\[u\:(.+?)\]~is',
				'~\[/u\:(.+?)\]~is',
				'~\[url\:(.+?)\]~is',
				'~\[/url\:(.+?)\]~is',
				'~\[url=(.+?)\:(.+?)\]~is',
				'~\[/url\:(.+?)\]~is',
				'~\<a(.+?) href="(.+?)">(.+?)</a>~is',
				'~\[img\:(.+?)\]~is',
				'~\[/img\:(.+?)\]~is',
				'~\[size=(.+?)\:(.+?)\]~is',
				'~\[/size\:(.+?)?\]~is',
				'~\[color=(.+?)\:(.+?)\]~is',
				'~\[/color\:(.+?)\]~is',
				'~\[code=(.+?)\:(.+?)\]~is',
				'~\[code\:(.+?)\]~is',
				'~\[/code\:(.+?)\]~is',
				'~\[list=(.+?)\:(.+?)\]~is',
				'~\[list\:(.+?)\]~is',
				'~\[/list\:(.+?)\]~is',
				'~\[\*\:(.+?)\]~is',
				'~\[/\*\:(.+?)\]~is',
				'~\<img src=\"{SMILIES_PATH}/(.+?)\" alt=\"(.+?)\" title=\"(.+?)\" /\>~is',
				'~<img src="{SMILIES_PATH}/(.+?)" alt="(.+?)" title="(.+?)" />~is',
				'~\[attachment=(.+?):(.+?)?\]~is',
				'~\[/attachment:(.+?)?\]~is',
			),
			array(
				'[quote author="$1"]',
				'[quote]',
				'[/quote]',
				'[b]',
				'[/b]',
				'[i]',
				'[/i]',
				'[u]',
				'[/u]',
				'[url]',
				'[/url]',
				'[url=$1]',
				'[/url]',
				'[url=$2]$3[/url]',
				'[img]',
				'[/img]',
				'[size=' . percent_to_px("\1") . 'px]',
				'[/size]',
				'[color=$1]',
				'[/color]',
				'[code=$1]',
				'[code]',
				'[/code]',
				'[list type=$1]',
				'[list]',
				'[/list]',
				'[li]',
				'[/li]',
				'$2',//is this still right??? it is $3 in the forum post...
				'$2',
				'[attachment=$1]',
				'[/attachment]',
			), $message);

		$message = preg_replace('~\[attachment=(\d+?)\]([^\[]*)\[/attachment\]~is', "[attachment=" . '\1' . "]", $message);
			
		$message = preg_replace('~\[size=(.+?)px\]~is', "[size=" . ('\1' > '99' ? 99 : '"\1"') . "px]", $message);

		$message = strtr($message, array(
			'[list type=1]' => '[list type=decimal]',
			'[list type=a]' => '[list type=lower-alpha]',
		));
		$message = stripslashes($message);

		return $message;
	} 

	function get_elkarte_config($to_prefix,$variable){
		$rows = $this->db->query("
			SELECT value
			FROM {$to_prefix}settings
			WHERE variable = '{$variable}'
			LIMIT 1");
		$result=$rows->fetch_all(MYSQLI_ASSOC);
		if(count($result)==0)
			return false;
		
		return $result[0]['value'];	
	}
	function get_phpbb_config($from_prefix,$variable){

		$rows=$this->db->query("
		SELECT config_value
		FROM {$from_prefix}config
		WHERE config_name = '{$variable}'
		LIMIT 1");
		$result=$rows->fetch_all(MYSQLI_ASSOC);
		if(count($result)==0)
			return false;
		
		return $result[0]['config_value'];		
	}	
	

	
	
}

		/**
	 * Add slashes recursively...
	 *
	 * @param array $var
	 *
	 * @return array
	 */
	function addslashes_recursive($var)
	{
		if (!is_array($var))
		{
			return addslashes($var);
		}
		else
		{
			foreach ($var as $k => $v)
			{
				$var[$k] = addslashes_recursive($v);
			}

			return $var;
		}
	}
	
	// Utility functions specific to phpbb

/**
 * @param int $percent
 *
 * @return int
 */
function percent_to_px($percent)
{
	return intval(11 * (intval($percent) / 100.0));
}

